﻿default dungeon_zoom = 3
default dungeon_padding = 8

init python:
    from copy import copy
    class dungeon_card:
        def __init__(self, image, name, health = 0, damage = 0, type = "weapon"):
            self.image = image
            self.name = name
            self.visible = True
            self.parent = None
            self.health = health
            self.damage = damage
            self.type = type
    class dungeon_player(renpy.Displayable):
        def __init__(self, image, name, health = 13, damage = 1):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.name = name
            self.x_index = 1
            self.y_index = 1

            self.width = (64 * dungeon_zoom) + dungeon_padding * 2
            self.height = (96 * dungeon_zoom) + dungeon_padding * 2
            self.x = (self.x_index * self.width) + dungeon_padding
            self.y = (self.y_index * self.height) + dungeon_padding
            self.x_target = self.x
            self.y_target = self.y

            self.rotation = 0
            self.rotation_direction = 1

            self.visible = True
            self.parent = None
            self.max_health = health
            self.max_damage = damage
            self.health = health
            self.damage = damage
        def move(self, y, x):
            if not x is None:
                self.x_index = x
                self.x_target = (x * self.width) + dungeon_padding
            if not y is None:
                self.y_index = y
                self.y_target = (y * self.height) + dungeon_padding
        def attack(self, y, x):
            if not x is None:
                x = (self.x_index + x)/2
                self.x_target = (x * self.width) + dungeon_padding
            if not y is None:
                y = (self.y_index + y)/2
                self.y_target = (y * self.height) + dungeon_padding
        def settle(self):
            self.x_target = (self.x_index * self.width) + dungeon_padding
            self.y_target = (self.y_index * self.height) + dungeon_padding

        def render(self,width,height,st,at):
            if self.visible:
                # card = dungeon_card(self.image, self.name, 1, 1, "player"),
                # card_render = renpy.get_screen("card_dungeon_card")
                img = Composite(
                    (64*dungeon_zoom,96*dungeon_zoom),
                    (0,0), "card_base",
                    (16*dungeon_zoom,16*dungeon_zoom), self.image,
                    (3*dungeon_zoom,4*dungeon_zoom), Text(str(self.health), color = "#ac3232", font = "card dungeon/tall pixels 2x.ttf", size = 24),
                    (53*dungeon_zoom,4*dungeon_zoom), Text(str(self.damage), color = "#9badb7", font = "card dungeon/tall pixels 2x.ttf", size = 24),
                )
                walking = False
                if self.x < self.x_target:
                    self.x += 20
                    walking = True
                elif self.x > self.x_target:
                    self.x += -20
                    walking = True
                if self.y < self.y_target:
                    self.y += 20
                    walking = True
                elif self.y > self.y_target:
                    self.y += -20
                    walking = True
                if distance(self.x, self.x_target) < 20:
                    self.x = self.x_target
                if distance(self.y, self.y_target) < 20:
                    self.y = self.y_target
                if walking:
                    self.rotation += self.rotation_direction
                    if self.rotation > 6:
                        self.rotation_direction = -2
                    elif self.rotation < -6:
                        self.rotation_direction = 2
                else:
                    self.rotation = 0
                    self.settle()
                t = Transform(img, rotate = self.rotation, rotate_pad = False)
                child_render = renpy.render(t, width, height, st, at)
                cw, ch = child_render.get_size()

                rv = renpy.Render(cw, ch)
                rv.blit(child_render, (self.x, self.y))

                renpy.redraw(self, 0)
                return rv
            else:
                return renpy.Render(0,0)

    def get_surrounds(lst, y, x):
        y_len = len(lst)-1
        x_len = len(lst[0])-1

        up = down = left = right = None

        if not y-1 < 0:
            up = lst[y-1][x]
        if not y-1 > y_len:
            down = lst[y-1][x]
        if not x-1 < 0:
            left = lst[y][x-1]
        if not x-1 > x_len:
            right = lst[y][x+1]
        
        return {
            "up": up,
            "down": down,
            "left": left,
            "right": right,
        }
    def get_grid_empties(lst):
        r = []
        for y,j in enumerate(lst):
            for x,i in enumerate(j):
                if i is None:
                    r.append((y,x))
        return r
    class dungeon_class:
        def __init__(self, cards):
            self.cards = cards
            self.enemies = [i for i in cards if i.type is "enemy"]
            self.field = []
            self.player = dungeon_player("card_player", "Blondie")
            self.generate()
            self.state = "playing"
            self.cash = 0
            self.xo = 0
            self.yo = 0
        def restart(self):
            self.generate()
            self.player.move(1,1)
            self.state = "playing"
            self.player.health = self.player.max_health
        def add_column(self):
            for row in self.field:
                card = copy(random.choice(self.cards))
                row.append(card)
        def spawn_loot(self, y, x):
            for y_offset in [-1, 0, 1]:
                for x_offset in [-1, 0, 1]:
                    try:
                        if x or y:
                            if self.field[y + y_offset][x + x_offset] is None:
                                item = copy(random.choice(self.cards))
                                if item.type in ["enemy"]:
                                    item = None
                                self.field[y + y_offset][x + x_offset] = item
                    except:
                        pass
        def spawn_enemies(self):
            empties = get_grid_empties(self.field)
            random.shuffle(empties)
            empties = empties[:3]
            for i in empties:
                self.field[i[0]][i[1]] = copy(random.choice(self.enemies))

            
        def move(self, y, x):
            x = self.player.x_index + x
            y = self.player.y_index + y
            if y < 0 or y > len(self.field)-1:
                return
            if x < 0 or x > len(self.field[0])-1:
                return
            target = self.field[y][x]
            self.clicked(target, y, x)

        def clicked(self, target, yi, xi):
            if self.state in ["dead", "waiting"]:
                return
            if distance(self.player.x_index, xi) > 1 or distance(self.player.y_index, yi) > 1:
                return
            x = xi
            y = yi
            if self.player.x_index == xi:
                x = None
            if self.player.y_index == yi:
                y = None
            if x != None and y != None:
                return

            if target and target.type is "mimic":
                target.image = "card_mimic"
                target.name = "mimic"
                target.type = "enemy"

            # self.set_offsets(yi, xi)
            if target is None or target.type in ["empty"]:
                self.player.move(y, x)
                renpy.play("card dungeon/move.ogg", "sound")
                return
            elif target.type is "weapon":
                self.player.move(y, x)
                renpy.play("card dungeon/weapon.ogg", "sound")
                return
            elif target.type is "cash":
                self.field[yi][xi] = None
                self.cash += 1
                self.player.move(y, x)
                renpy.play("card dungeon/coin.ogg", "sound")
                return
            elif target.type is "potion":
                self.player.move(y, x)
                self.player.health = min(self.player.health + target.health, self.player.max_health)
                self.field[yi][xi] = None
                renpy.play("card dungeon/potion.ogg", "sound")
                return
            elif target.type is "elixir":
                self.player.move(y, x)
                self.player.max_health += target.health
                self.player.health += target.health
                self.field[yi][xi] = None
                renpy.play("card dungeon/potion.ogg", "sound")
                return
            elif target.type is "food":
                self.player.move(y, x)
                self.player.health = min(self.player.health + target.health, self.player.max_health)
                self.field[yi][xi] = None
                renpy.play("card dungeon/bite.ogg", "sound")
                return
            elif target.type is "chest":
                self.player.move(y, x)
                self.spawn_loot(yi, xi)
                target.type = "empty"
                target.image = "card_chest_opened"
                renpy.play("card dungeon/bite.ogg", "sound")
                return
            elif target.type is "backpack":
                self.player.move(y, x)
                self.spawn_loot(yi, xi)
                self.field[yi][xi] = None
                renpy.play("card dungeon/bite.ogg", "sound")
                return
            elif target.type is "campfire":
                self.player.move(y, x)
                self.player.health = self.player.max_health
                self.spawn_enemies()
                renpy.play("card dungeon/bite.ogg", "sound")
                return
            elif target.type is "door":
                self.player.move(y, x)
                self.state = "win"
                renpy.play("card dungeon/bite.ogg", "sound")
                return

            elif target.type == "enemy":
                current = self.field[self.player.y_index][self.player.x_index]
                weapon_damage = 0
                if current and current.type == "weapon":
                    weapon_damage = current.damage

                self.player.health -= target.damage
                target.health -= self.player.damage + weapon_damage
                self.player.attack(y, x)
                renpy.play("card dungeon/attack.ogg", "sound")
                if target.health < 1:
                    self.field[yi][xi] = None
                    self.show_a_door()
                    renpy.play("card dungeon/kill.ogg", "sfx1")
                if self.player.health < 1:
                    self.player.health = 0
                    self.state = "dead"
        def show_a_door(self):
            for j in self.field:
                for i in j:
                    if i and i.type is "enemy":
                        return
            
            empties = get_grid_empties(self.field)
            spot = random.choice(empties)
            self.field[spot[0]][spot[1]] = dungeon_card("card_door", "Door", 0, 0, "door")
        def set_offsets(self, y, x):
            self.xo = -(x*64)*dungeon_zoom
            self.yo = -(y*96)*dungeon_zoom
        def generate(self):
            field = []
            for j in range(3):
                row = []
                for i in range(4):
                    card = copy(random.choice(self.cards))
                    row.append(card)
                field.append(row)
            field[1][1] = None
            self.field = field
init:
    image card_base:
        Crop((0,0,64,96), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_player:
        Crop((64,0,32,64), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_hamzilla:
        Crop((96,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_kithulu:
        Crop((128,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_demon_stump:
        Crop((160,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_bite_stool:
        Crop((192,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_chest_closed:
        Crop((224,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_fat_chick:
        Crop((96,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_punch_rat:
        Crop((128,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_eye_fly:
        Crop((160,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_mimic:
        Crop((192,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_chest_opened:
        Crop((224,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_cake:
        Crop((64,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_meat:
        Crop((96,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_potion:
        Crop((128,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_elixir:
        Crop((160,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_campfire:
        Crop((192,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_backpack:
        Crop((224,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_knife:
        Crop((0,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_axe:
        Crop((32,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_hammer:
        Crop((64,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_staff:
        Crop((96,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_cutlass:
        Crop((128,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_sword:
        Crop((160,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_horseshoe:
        Crop((64,128,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_gold:
        Crop((96,128,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_door:
        Crop((128,128,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
transform card_dungeon_offset(x, y):
    ease 5 xoffset x yoffset y
transform card_dungeon_highlight:
    on idle:
        ease .2 additive .0
    on hover:
        ease .2 additive .2
style card_dungeon_dialogue_frame:
    background "#150b1fe4"
    padding (20,20)
style card_dungeon_dialogue_button:
    background "#290e44e4"
    padding (20,20)
style card_dungeon_dialogue_text:
    font "card dungeon/tall pixels 2x.ttf"
    size 28
style card_dungeon_text:
    font "card dungeon/tall pixels 2x.ttf"
    size 24
screen card_dungeon_screen():
    default g = dungeon_class(
        [
            dungeon_card("card_hamzilla", "HamZilla", 1, 1, "enemy"),
            dungeon_card("card_kithulu", "Kithullu", 1, 2, "enemy"),
            dungeon_card("card_bite_stool", "Bite stool", 2, 1, "enemy"),
            dungeon_card("card_demon_stump", "Demon stump", 5, 1, "enemy"),
            dungeon_card("card_fat_chick", "Fat chick", 2, 3, "enemy"),
            dungeon_card("card_punch_rat", "Punch rat", 3, 3, "enemy"),
            dungeon_card("card_eye_fly", "Eye fly", 2, 4, "enemy"),
            dungeon_card("card_chest_closed", "Chest", 4, 3, "mimic"),
            dungeon_card("card_chest_closed", "Chest", 0, 0, "chest"),
            dungeon_card("card_cake", "Cake", 1, 2, "food"),
            dungeon_card("card_meat", "Meat", 3, 3, "food"),
            dungeon_card("card_potion", "Potion", 5, 0, "potion"),
            dungeon_card("card_elixir", "Elixir", 5, 0, "elixir"),
            dungeon_card("card_campfire", "Campfire", 0, 0, "campfire"),
            dungeon_card("card_backpack", "Backpack", 0, 0, "backpack"),
            dungeon_card("card_knife", "Knife", 0, 1),
            dungeon_card("card_hammer", "Hammer", 0, 2),
            dungeon_card("card_staff", "Staff", 1, 3),
            dungeon_card("card_axe", "Axe", 0, 4),
            dungeon_card("card_cutlass", "Cutlass",0 ,5),
            dungeon_card("card_sword", "Sword", 0, 6),
            dungeon_card("card_horseshoe", "Horseshoe", 0, 0, "cash"),
            dungeon_card("card_gold", "Gold", 0, 0, "cash"),
        ]
    )
    style_prefix "card_dungeon"
    key "w" action Function(g.move, -1, 0)
    key "a" action Function(g.move, 0, -1)
    key "s" action Function(g.move, 1, 0)
    key "d" action Function(g.move, 0, 1)
    fixed fit_first True:
        align .5,.5
        # at card_dungeon_offset(g.xo, g.yo)
        vbox:
            spacing 0
            for y,j in enumerate(g.field):
                hbox:
                    spacing 0
                    for x,i in enumerate(j):
                        button:
                            padding dungeon_padding,dungeon_padding
                            action Function(g.clicked, i, y, x)
                            at card_dungeon_highlight
                            use card_dungeon_card(i)
        add g.player align .0,.0

    if g.state is "dead":
        frame:
            style_prefix "card_dungeon_dialogue"
            align (0.5, 0.5)
            vbox:
                spacing 10
                text "You have died."
                hbox:
                    spacing 10
                    button:
                        text "Restart"
                        action Function(g.restart)
                    button:
                        text "Exit"
                        action Return()
    if g.state is "win":
        frame:
            style_prefix "card_dungeon_dialogue"
            align (0.5, 0.5)
            vbox:
                spacing 10
                text "You have survived."
                hbox:
                    spacing 10
                    button:
                        text "Again"
                        action Function(g.restart)
                    button:
                        text "Exit"
                        action Return()

    hbox:
        align (1.0,.0)
        text str(g.cash) size 30 align (0.5, 0.5)
        add "card_gold" align (0.5, 0.5)

    # Tests
    vbox:
        text str(g.xo)
        button:
            text "Reset"
            action Function(g.restart)
        button:
            text "Add column"
            action Function(g.add_column)
        button:
            text "Exit"
            action Return()

screen card_dungeon_card(i = None):
    fixed fit_first True:
        add "card_base"
        if i:
            add i.image align .5,1.0 yoffset -16*dungeon_zoom
            fixed:
                xysize 39,45 align .0,.0
                text str(i.health) align .5,.5 color "#ac3232"
            fixed:
                xysize 39,45 align 1.0,.0
                text str(i.damage) align .5,.5 color "#9badb7"
            text str(i.name) align .5,1.0 color "#9badb7" yoffset -10

label card_dungeon_example:
    show bg room with dissolve
    call screen card_dungeon_screen
